"""SegFormer model preset configurations."""

presets = {
    "segformer_b0_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB0 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 3719027,
            "path": "segformer_b0",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b0_ade20k_512/3",
    },
    "segformer_b1_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB1 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 13682643,
            "path": "segformer_b1",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b1_ade20k_512/5",
    },
    "segformer_b2_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB2 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 24727507,
            "path": "segformer_b2",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b2_ade20k_512/3",
    },
    "segformer_b3_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB3 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 44603347,
            "path": "segformer_b3",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b3_ade20k_512/3",
    },
    "segformer_b4_ade20k_512": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB4 backbone fine-tuned on ADE20k in "
                "512x512 resolution."
            ),
            "params": 61373907,
            "path": "segformer_b4",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b4_ade20k_512/3",
    },
    "segformer_b5_ade20k_640": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB5 backbone fine-tuned on ADE20k in "
                "640x640 resolution."
            ),
            "params": 81974227,
            "path": "segformer_b5",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b5_ade20k_640/3",
    },
    "segformer_b0_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB0 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 3719027,
            "path": "segformer_b0",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b0_cityscapes_1024/3",
    },
    "segformer_b1_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB1 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 13682643,
            "path": "segformer_b1",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b1_ade20k_512/1",
    },
    "segformer_b2_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB2 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 24727507,
            "path": "segformer_b2",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b2_cityscapes_1024/3",
    },
    "segformer_b3_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB3 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 44603347,
            "path": "segformer_b3",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b3_cityscapes_1024/3",
    },
    "segformer_b4_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB4 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 61373907,
            "path": "segformer_b4",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b4_cityscapes_1024/3",
    },
    "segformer_b5_cityscapes_1024": {
        "metadata": {
            "description": (
                "SegFormer model with MiTB5 backbone fine-tuned on Cityscapes "
                "in 1024x1024 resolution."
            ),
            "params": 81974227,
            "path": "segformer_b5",
        },
        "kaggle_handle": "kaggle://keras/segformer/keras/segformer_b5_cityscapes_1024/3",
    },
}
