"""DO NOT EDIT.

This file was autogenerated. Do not edit it by hand,
since your modifications would be overwritten.
"""

from keras_hub.src.utils.coco.coco_utils import (
    coco_id_to_name as coco_id_to_name,
)
from keras_hub.src.utils.coco.coco_utils import (
    coco_name_to_id as coco_name_to_id,
)
from keras_hub.src.utils.imagenet.imagenet_utils import (
    decode_imagenet_predictions as decode_imagenet_predictions,
)
from keras_hub.src.utils.imagenet.imagenet_utils import (
    imagenet_id_to_name as imagenet_id_to_name,
)
from keras_hub.src.utils.imagenet.imagenet_utils import (
    imagenet_name_to_id as imagenet_name_to_id,
)
